module constants

!-----------------------------------------------------------------------------------------------!
!       Constants module for CCS/2L-CCS simulations.                                        !
!                                                                                               !
! Initialises variaous parameters                                                               !
!-----------------------------------------------------------------------------------------------!

!----------------------------------Parameters to be read at runtime----------------------------------!
  integer::dof                  !Number of degrees of freedom
  integer::sys_dof              !Number of degrees of freedom for the system (in 2L-CCS / =1 for CCS)
  integer::nbv                  !Number of basis vectors in a single configuration (=1 for CCS)
  integer::nconf                !Number of configurations
  real(kind=8)::dt              !Timestep
  real(kind=8)::time_end        !End time
  real(kind=8)::cmprss          !Compression parameter for sampling swarm (and bath in pancake sampling)
  real(kind=8)::sys_cmprss      !Compression parameter for sampling system in pancake sampling
  character(len=5)::method      !Method: CCS or 2LCCS
  character(len=2)::qsystem      !Hamiltonian, free particle [FP], harmonic potential [HP], morse potential [MP], asymmetric double well [AS] or second quantized double well [2Q]
  character(len=10)::sampling   !Sampling method, currently only pancake or swarm
  integer::as_upper_well_dim    !Number of basis vectors in upper well for [AS]
  integer::bath_sampling        !Type of sampling to be adopted by the bath
  character(len=1)::bsout       !Write basis to output file every 1 a.u. Y/N
  character(len=1)::readbs      !Read basis from file Y/N
  character(len=1)::reporting   !Report time dependence of trajectories and amplitudes Y/N
  integer::n_prtcls             !Number of bath particles in ground state for second quantization
  real(kind=8)::n_prtcls_fact   !Factorial of the above (real as the integer value cannot be held in memory)
  character(len=1000)::filename !Name appended to output files

!-----------------------------------Hamiltonian parameters-------------------------------------------!
  real(kind=8)::max_norm               !Max norm and min norm are set in readparams() based on
  real(kind=8)::min_norm               !which system is being studied
!--------------------------------------1)Free Particle-----------------------------------------------!
  real(kind=8),parameter::fp_max_norm=1.001d0
  real(kind=8),parameter::fp_min_norm=0.999d0
!------------------------------------2)Harmonic Potential--------------------------------------------!
  real(kind=8),parameter::hp_max_norm=1.001d0
  real(kind=8),parameter::hp_min_norm=0.999d0
!-------------------------------------3)Morse Potential----------------------------------------------!
  real(kind=8),parameter::mp_max_norm=1.001d0
  real(kind=8),parameter::mp_min_norm=0.999d0
  real(kind=8),parameter::mp_diss_en=10.25d0
  real(kind=8),parameter::mp_well_param=0.2209d0
!-----------------------4)Asymmetric Double Well and Second Quantized Version------------------------!
  real(kind=8),parameter::as_max_norm=1.01d0
  real(kind=8),parameter::as_min_norm=0.990d0
  real(kind=8),parameter::as_eta=1.3544d0
  real(kind=8),parameter::as_lambda=0.10d0
!--------------------------------------5)Bose-Einstein Condensate------------------------------------!
  real(kind=8),parameter::bc_max_norm=1.001d0
  real(kind=8),parameter::bc_min_norm=0.999d0
  real(kind=8),parameter::bc_lambda=0.001d0                         !Two body interaction
  real(kind=8),allocatable::bc_W_mnop(:,:,:,:)                !The <m,n|dirac_delta|o,p> matrix
  real(kind=8),allocatable::hermite_coeff(:,:)                !Array of Hermite coefficients, (power of x, order of Hermite)
!----------------------------------------------------------------------------------------------------!

  real(kind=8),parameter::sigq=1/sqrt(2.d0)                !Width of real part of z distribution
  real(kind=8),parameter::sigp=1/sqrt(2.d0)                !Width of imag part of z distribution
  complex(kind=8),parameter::im=(0.d0,1.d0)                !Imaginary number
  integer,parameter::acf_ccf_flag=2                        !Flag for autocorrelation function (1) or cross-correlation function (2)
  real(kind=8),parameter::Pi=4.d0*atan(1.d0)               !Pi

  real(kind=8),external :: ZBQLNOR                         !The normally distributed random number external function
  real(kind=8),external :: ZBQLGAM                         !The gamma distributed random number external function

end module constants
