program main

use constants; use initialise; use auxiliary; use basis; use outputs; use hamiltonian; use propagation

  implicit none

!----------------------------------Time dependent CCS parameters----------------------------------!
  complex(kind=8),allocatable::z(:,:,:)         !coherent state array (dof,nbv,nconf)
  complex(kind=8),allocatable::d(:,:)           !factored single config amplitude (nbv,nconf)
  complex(kind=8),allocatable::bigD(:)          !multi config amplitude (nconf)
  real(kind=8),allocatable::s(:,:)              !action (nvb,nconf)
!------------------------------------------Working matrices---------------------------------------!
  complex(kind=8),allocatable::H(:,:,:,:)       !Hamiltonian (nbv,nbv,nconf,nconf)
  complex(kind=8),allocatable::ovrlp(:,:,:,:)   !Overlap matrix (nbv,nbv,nconf,nconf) <z1|z2>
  complex(kind=8),allocatable::density_fock(:,:)!Density matrix (dof,dof)
!------------------------------------------Working variables--------------------------------------!
  real(kind=8)::t=0.d0                          !time
  real(kind=8)::norm                            !norm of basis
  complex(kind=8)::acf_ccf                      !acf or ccf (depending on flag given in const.f90)
!------------------------------------------Temp variables-----------------------------------------!
  integer::restart=1                            !loop controller to check basis is within required norm
  integer::norm_fail_counter=1                  !counter which stops the program if a basis has been attempted to be created too many times without success
!----------------------------------------Permanent variables--------------------------------------!
  complex(kind=8),allocatable::psi0(:)        !Initial wavepacket (dof)
!-------------------------------------------------------------------------------------------------!
!------------------------------------------BEGIN PROGRAM------------------------------------------!

  call readparams()                                               !Read parameters from input file(s)

  call alarrays(z,d,bigD,s,H,ovrlp,psi0,density_fock)             !Allocate arrays

  call gen_filename()                                             !Generate the filename to append to the output files

  if (qsystem.eq."BC") then
    call calc_hermite_coeff!(hermite_coeff)                       !Calculate the hermite polynomial coefficients to store in global array

    call calc_bc_W_mnop                                           !Calculate the dirac delta overlap matrix for bose einstein condensate to store in global array
  end if

  call get_psi0(psi0)                                             !Get the value of the initial wavepacket, psi0

  if (readbs.eq."N") then !If the basis is not being read from a previous outbs file then....

    call gen_seed()                                               !Generate the seed for the random number generator used in the Monte-Carlo sampling of z

    do while (restart.eq.1)                                       !Regenerate basis set if norm is not between limits

      call gen_basis(z,d,bigD,ovrlp,psi0)                         !Generate the basis of coherent states and amplitudes
 
      norm = abs(calc_norm(z,d,bigD,s,ovrlp))                     !Calculate norm
      print*,norm                                                 !Print norm to screen to check whether appropriate parameters have been chosen (bad norm indicates not)
      if ((norm.gt.min_norm).and.(norm.lt.max_norm)) then         !Check initial norm is within appropriate limits
        restart=0
      else
        restart=1
        norm_fail_counter=norm_fail_counter+1                     !Increment the norm_fail_counter
        if (norm_fail_counter.gt.1000) then                       !If norm_fail_counter exceeds 1000 attemps to make a basis without success, stop the program
          print*,"Could not form a basis"
          stop
        end if
      end if

    end do                                                       !End of basis regeneration loop

    call output_heads()                                          !Make headers for output file(s)

    if (qsystem.eq."BC") then !If using BC system then calculate density matrix, otherwise calculate initial ACF/CCF
      call calc_density_fock(z,bigD,s,density_fock,ovrlp)        !Calculate the initial value of the density in the fock basis
    else
      acf_ccf = calc_acf_ccf(z,d,bigD,s,ovrlp,psi0)              !Calculate initial ACF/CCF
    end if

    call write_outputs(t,norm,acf_ccf,z,s,d,bigD,ovrlp,density_fock)   !Write initial outputs to file

  else if (readbs.eq."Y") then

    call read_basis(t,z,s,d,bigD)                                !Read basis from file outbs.out

    call calc_ovrlp(ovrlp,z)                                     !calculate overlap <z1|z2>

  end if

  do while (t.lt.time_end)                                       !Propagation loop

    t = t+dt                                                     !Increment time

    call rk4_prop(z,d,bigD,s,ovrlp,H)                            !Take a Runge-Kutta 4 propagation step

    call calc_ovrlp(ovrlp,z)                                     !update overlap <z1|z2>

    if (qsystem.eq."BC") then !If using BC system then calculate density matrix, otherwise calculate ACF/CCF
      call calc_density_fock(z,bigD,s,density_fock,ovrlp)        !Calculate the value of the density in the fock basis
    else
      acf_ccf = calc_acf_ccf(z,d,bigD,s,ovrlp,psi0)              !Calculate ACF/CCF
    end if

    norm = abs(calc_norm(z,d,bigD,s,ovrlp))                      !Calculate norm

    call write_outputs(t,norm,acf_ccf,z,s,d,bigD,ovrlp,density_fock)   !Write outputs to file

    if (norm.gt.1.5d0) then                                      !Check if norm is exploding
      print*,"norm is exploding, exit",norm                      !If norm is exploding finish simulation
      stop
    end if

  end do                                                         !End of propagation loop

  stop

end program main
  

