module outputs

use constants; use auxiliary;

!-----------------------------------------------------------------------------------------------!
!       Module to produce outputs CCS/2L-CCS simulation.                                        !
!                                                                                               !
! Contains subroutines to:                                                                      !
! 1) Make output filename                                                                       !
! 2) Make headers for output files                                                              !
! 3) Write values to output files                                                               !
!-----------------------------------------------------------------------------------------------!

contains

!-----------------------------------------------------------------------------------------------!
subroutine gen_filename()

  implicit none

  character(len=10)::nconf_char,nbv_char,dof_char,cmprss_char,plot_low_char,plot_high_char,ndof_char
  character(len=100)::cmprss_char_tmp

  write(nconf_char,"(i0)")nconf
  write(nbv_char,"(i0)")nbv
  write(ndof_char,"(i0)")dof
  write(cmprss_char_tmp,"(ES100.10E2)")real(cmprss,kind=4) 
 
  cmprss_char = cmprss_char_tmp((101-len(trim(adjustl(cmprss_char_tmp)))):(104-len(trim(adjustl(cmprss_char_tmp)))))&
                                 //cmprss_char_tmp(97:100)

  filename = trim(method)//"_"//trim(qsystem)//"_"//trim(ndof_char)//"dof_"//trim(nconf_char)//"conf_"&
             //trim(nbv_char)//"bv_"//trim(cmprss_char)//"cmprss"

  return

end subroutine gen_filename

!-----------------------------------------------------------------------------------------------!

subroutine output_heads()

  implicit none

  character(len=10)::dof_char,plot_low_char,plot_high_char
  character(len=5)::density_gif_char
  character(len=100000)::plot_char,plot_char_temp
  integer::m,i,k,filenum

  !Make filename to be appended to output files (filename is global so only change here!)
  
  write(density_gif_char,"(i0)")(int(time_end/0.1d0)+1)


  if (qsystem.eq."BC") then !make density output file
    open(296,file="../out/density_"//trim(filename)//".out",status="unknown")
    write(296,*) "t          x              p(x)           p(x) Norm"
    write(296,*) ""
    close(296)

    !make part_num + norm + variance output file
    open(297,file="../out/BC_"//trim(filename)//".out",status="unknown")
    write(297,*) "t          Norm            Part Num(Calcd)           Part Num(trace)           <Q> Var        <Q> Var Norm"
    write(297,*) ""
    close(297)
  else
    open(290,file="../out/ccf_"//trim(filename)//".out",status="unknown")
    write(290,*)"Method=",trim(method),", System=",qsystem,", nconf=",nconf,", nbv=",nbv,", dof=",dof," cmprss=",real(cmprss,kind=4),&
                " sys_cmprss=",real(sys_cmprss,kind=4)," n_upper_well=",as_upper_well_dim," bath sampling type=",bath_sampling
    write(290,*) "Time          Re(CCF)          Im(CCF)       |CCF|             Norm           Part Num"
    write(290,*) ""
    close(290)
  end if

  !!!!!!!!!!!!!!!!!!!! Create gnuplot output file for ccf !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  open(292,file='../out/plotccf.gnu')

  write(292,*)'set terminal postscript eps enhanced color size 5.00,2.00'
  write(292,*)'set output '//'"ccf_'//trim(filename)//'.eps"'
  write(292,*)'set xlabel "Time (au)"'
  write(292,*)'set ylabel "CCF"'
  write(292,*)'set xrange [0:120]'
  write(292,*)'unset key'
  write(292,*)'plot "ccf_'//trim(filename)//'.out"'//' u 1:2 w l lt -1'
  close(292)

  !!!!!!!!!!!!!!!!!!!! Create gnuplot output file for ft !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  open(293,file='../out/plotft.gnu')

  write(293,*)'set terminal postscript eps enhanced color size 5.00,4.00'
  write(293,*)'set output '//'"ft_'//trim(filename)//'.eps"'
  write(293,*)'set xlabel "{/Symbol w}"'
  write(293,*)'set ylabel "{/Helvetica-Italic I}({/Symbol w})"'
  write(293,*)'set xrange [7:14]'
  write(293,*)'unset key'
  write(293,*)'plot "ft_'//trim(filename)//'.out"'//' u 1:2 w l lt -1'
  close(293)

  !!!!!!!!!!!!!!!!!!!! Create gnuplot output file for density map if BC!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  if (qsystem.eq."BC") then

    open(294,file='../out/plot_density_map.gnu')

    write(294,*)'set terminal pngcairo enhanced size 800,800'
    write(294,*)'set output '//'"density_map_'//trim(filename)//'.png"'
    write(294,*)'unset key'
    write(294,*)'set cbrange [0.01:]'
    write(294,*)'set xlabel "Space (x)"'
    write(294,*)'set ylabel "Time (t)"'
    write(294,*)'unset colorbox'
    write(294,*)'set pm3d map'
    write(294,*)'set pm3d interpolate 5,5 flush begin ftriangles nohidden3d corners2color mean'
    write(294,*)'splot "density_'//trim(filename)//'.out"'//' u 2:1:3'
    close(294)

  !!!!!!!!!!!!!!!!!!!! Create gnuplot output file for density gif if BC!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    open(295,file='../out/plot_density_gif.gnu')

    write(295,*)'set terminal gif animate delay 10'
    write(295,*)'set output '//'"density_animation_'//trim(filename)//'.gif"'
    write(295,*)'set xlabel "Space (x)"'
    write(295,*)'set ylabel "Density rho(x)"'
    write(295,*)'stats "density_'//trim(filename)//'.out" u 2:3 nooutput'
    write(295,*)'set yrange [0:(STATS_max_y+STATS_max_y*0.1)]'
    write(295,*)'do for [i=1:'//trim(density_gif_char)//'] {'
    write(295,*)'plot "density_'//trim(filename)//'.out" every :::i::i u 2:3 w filledcurve title sprintf("t=%2.3f",(i-1)*0.1)'
    write(295,*)'}'
    close(295)

  end if

  !If reporting time dependence of all trajectories, action and amplitudes, make output files

  if (reporting.eq.'Y') then     !Report time dependence of trajectories and amplitudes over whole simulation

    call system("mkdir ../out/dump/"//trim(filename))        !Make folder for the analysis dump for this simulation

    open(291,file='../out/dump/'//trim(filename)//'/var_dump.out',status="unknown")   !Write header for output file
    write(291,*)"Method=",trim(method),", System=",qsystem,", nconf=",nconf,", nbv=",nbv,", dof=",dof," cmprss=",&
                real(cmprss,kind=4)," sys_cmprss=",real(sys_cmprss,kind=4)," n_upper_well=",as_upper_well_dim,&
                " bath sampling type=",bath_sampling
    write(291,*) "Time          Re(z(dof,)          Im(z)      s      Re(d)      Im(d)      Re(D)       Im(D)"
    write(291,*) ""
    close(291)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot outputs for trajectories!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    do m=1,dof

      do k=1,nconf !Create character for gnuplot file for plotting trajectories (im(z) vs re(z))
        do i=1,nbv
            write(plot_low_char,"(i0)")(m+1+dof*(i-1)+dof*nbv*(k-1))
            write(plot_high_char,"(i0)")(dof*nconf*nbv+m+1+dof*(i-1)+dof*nbv*(k-1))

            if ((i.eq.1).and.(k.eq.1)) then
              plot_char = 'plot "var_dump.out" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
            else
              plot_char_temp = plot_char
              plot_char = trim(plot_char_temp)//', "" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
            end if

        end do
      end do

      filenum=500+m
      write(dof_char,"(i0)")m        

      open(filenum,file='../out/dump/'//trim(filename)//'/plot_traj_dof'//trim(dof_char)//'.gnu')

      write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
      write(filenum,*)'set output '//'"traj_dof'//trim(dof_char)//'.eps"'
      write(filenum,*)'set xlabel "Re(z)"'
      write(filenum,*)'set ylabel "Im(z)"'
      write(filenum,*)'unset key'
      write(filenum,*) trim(plot_char)

      close(filenum)

    end do

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot output for action!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    do k=1,nconf !Create character for gnuplot file for plotting action vs time
      do i=1,nbv
          write(plot_high_char,"(i0)")(2*dof*nconf*nbv+1+i+nbv*(k-1))

          if ((i.eq.1).and.(k.eq.1)) then
            plot_char = 'plot "var_dump.out" u 1:'//trim(plot_high_char)//' w l'
          else
            plot_char_temp = plot_char
            plot_char = trim(plot_char_temp)//', "" u 1:'//trim(plot_high_char)//' w l'
          end if

      end do
    end do

    filenum=500+dof+1       

    open(filenum,file='../out/dump/'//trim(filename)//'/plot_action.gnu')

    write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
    write(filenum,*)'set output "action.eps"'
    write(filenum,*)'set xlabel "Time"'
    write(filenum,*)'set ylabel "Action"'
    write(filenum,*)'unset key'
    write(filenum,*) trim(plot_char)

    close(filenum)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot output for d amplitudes!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    do k=1,nconf !Create character for gnuplot file for plotting d amplitudes (im vs re)
      do i=1,nbv
          write(plot_low_char,"(i0)")(2*dof*nconf*nbv+nconf*nbv+1+i+nbv*(k-1))
          write(plot_high_char,"(i0)")(2*dof*nconf*nbv+2*nconf*nbv+1+i+nbv*(k-1))

          if ((i.eq.1).and.(k.eq.1)) then
            plot_char = 'plot "var_dump.out" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
          else
            plot_char_temp = plot_char
            plot_char = trim(plot_char_temp)//', "" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
          end if

      end do
    end do

    filenum=500+dof+2       

    open(filenum,file='../out/dump/'//trim(filename)//'/plot_d_amps.gnu')

    write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
    write(filenum,*)'set output "d_amps.eps"'
    write(filenum,*)'set xlabel "Re(d)"'
    write(filenum,*)'set ylabel "Im(d)"'
    write(filenum,*)'unset key'
    write(filenum,*) trim(plot_char)

    close(filenum)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot output for abs(d) amplitudes!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    do k=1,nconf !Create character for gnuplot file for plotting abs(d) amplitudes vs time
      do i=1,nbv
          write(plot_high_char,"(i0)")(2*dof*nconf*nbv+3*nconf*nbv+1+i+nbv*(k-1))

          if ((i.eq.1).and.(k.eq.1)) then
            plot_char = 'plot "var_dump.out" u 1:'//trim(plot_high_char)//' w l'
          else
            plot_char_temp = plot_char
            plot_char = trim(plot_char_temp)//', "" u 1:'//trim(plot_high_char)//' w l'
          end if

      end do
    end do

    filenum=500+dof+3       

    open(filenum,file='../out/dump/'//trim(filename)//'/plot_abs_d_amps.gnu')

    write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
    write(filenum,*)'set output "abs_d_amps.eps"'
    write(filenum,*)'set xlabel "Time"'
    write(filenum,*)'set ylabel "|d|"'
    write(filenum,*)'unset key'
    write(filenum,*) trim(plot_char)

    close(filenum)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot output for bigD amplitudes!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    do k=1,nconf !Create character for gnuplot file for plotting bigD amplitudes (im vs re)
        write(plot_low_char,"(i0)")(2*dof*nconf*nbv+4*nconf*nbv+1+k)
        write(plot_high_char,"(i0)")(2*dof*nconf*nbv+4*nconf*nbv+nconf+1+k)

        if (k.eq.1) then
          plot_char = 'plot "var_dump.out" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
        else
          plot_char_temp = plot_char
          plot_char = trim(plot_char_temp)//', "" u '//trim(plot_low_char)//':'//trim(plot_high_char)//' w l'
        end if
    end do

    filenum=500+dof+4       

    open(filenum,file='../out/dump/'//trim(filename)//'/plot_bigD_amps.gnu')

    write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
    write(filenum,*)'set output "bigD_amps.eps"'
    write(filenum,*)'set xlabel "Re(D)"'
    write(filenum,*)'set ylabel "Im(D)"'
    write(filenum,*)'unset key'
    write(filenum,*) trim(plot_char)

    close(filenum)

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!Create gnuplot output for abs(bigD) amplitudes!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    do k=1,nconf !Create character for gnuplot file for plotting abs(bigD) amplitudes vs time
        write(plot_high_char,"(i0)")(2*dof*nconf*nbv+4*nconf*nbv+2*nconf+1+k)

        if (k.eq.1) then
          plot_char = 'plot "var_dump.out" u 1:'//trim(plot_high_char)//' w l'
        else
          plot_char_temp = plot_char
          plot_char = trim(plot_char_temp)//', "" u 1:'//trim(plot_high_char)//' w l'
        end if
    end do

    filenum=500+dof+5       

    open(filenum,file='../out/dump/'//trim(filename)//'/plot_abs_bigD_amps.gnu')

    write(filenum,*)'set terminal postscript eps enhanced color solid size 5.00,4.00'
    write(filenum,*)'set output "abs_bigD_amps.eps"'
    write(filenum,*)'set xlabel "Time"'
    write(filenum,*)'set ylabel "|D|"'
    write(filenum,*)'unset key'
    write(filenum,*) trim(plot_char)

    close(filenum)

  end if

  return

end subroutine output_heads

!-----------------------------------------------------------------------------------------------!

subroutine write_outputs(t,norm,acf_ccf,z,s,d,bigD,ovrlp,density_fock) 

  implicit none
  
  real(kind=8),intent(in)::t
  real(kind=8),intent(in)::norm
  complex(kind=8),intent(in)::acf_ccf
  integer::m,i,k,l,x_counter
  complex(kind=8),intent(in)::z(:,:,:)         !coherent state array (dof,nbv,nconf)
  complex(kind=8),intent(in)::d(:,:)           !factored single config amplitude (nbv,nconf)
  complex(kind=8),intent(in)::bigD(:)          !multi config amplitude (nconf)
  real(kind=8),intent(in)::s(:,:)              !action (nvb,nconf) 
  complex(kind=8),intent(in)::ovrlp(:,:,:,:)   !Overlap matrix(nbv,nbv,nconf,nconf)
  complex(kind=8),intent(in)::density_fock(:,:)!Density matrix (dof,nconf)
  real(kind=8)::x
  real(kind=8)::dens_trace
  real(kind=8)::density_1_body

  dens_trace=0.d0
  do m=1,size(density_fock,2)
    dens_trace = dens_trace + density_fock(m,m)
  end do

  if (qsystem.eq."BC") then
    if (mod(nint(t/dt),(nint(0.1/dt))).eq.0) then
      open(296,file="../out/density_"//trim(filename)//".out",access="append")
      do x_counter=0,128
        x = -8.d0+x_counter/128.d0*16.d0
        density_1_body = real(calc_density_1_body(x,density_fock))
        write(296,*)real(t,kind=4),x,density_1_body,density_1_body/dens_trace
      end do
      write(296,*) ""
      close(296)
    end if

    open(297,file="../out/BC_"//trim(filename)//".out",status="old",access="append")
    write(297,*)real(t,kind=4),norm,calc_part_num(z,bigD,s,ovrlp),dens_trace,calc_pos_variance(density_fock,.false.)&
                ,calc_pos_variance(density_fock,.true.)
    close(297)
    
  else
    open(290,file="../out/ccf_"//trim(filename)//".out",status="old",access="append")
    write(290,*)t,real(acf_ccf),imag(acf_ccf),abs(acf_ccf),norm,calc_part_num(z,bigD,s,ovrlp)
    close(290)
  end if

  if (t.eq.0.d0) then !Output initial basis
    open(434,file="../out/basis_"//trim(filename)//".out")
    write(434,*) "re(z)          im(z)          dof            bv           conf"
    do k=1,size(z,3)
      do i=1,size(z,2)
        do m=1,size(z,1)
          write(434,*) real(z(m,i,k)),imag(z(m,i,k)),m,i,k
        end do
      end do
    end do
    close(434)
  end if

  if (t.ge.time_end) then !Output final basis
    open(435,file="../out/final_basis_"//trim(filename)//".out")
    write(435,*) "re(z)          im(z)          dof            bv           conf"
    do k=1,size(z,3)
      do i=1,size(z,2)
        do m=1,size(z,1)
          write(435,*) real(z(m,i,k)),imag(z(m,i,k)),m,i,k
        end do
      end do
    end do
    close(435)
  end if

  if ((reporting.eq.'Y').and.(mod(t,1.d0).lt.dt)) then     !Create dump file for outputting trajectories and amplitudes over the whole simulation every 0.1 a.u.

    open(291,file="../out/dump/"//trim(filename)//"/var_dump.out",status="old",access="append")
    write(291,*)t,real(z(:,:,:)),imag(z(:,:,:)),s(:,:),real(d(:,:)),imag(d(:,:)),abs(d(:,:)),real(bigD(:)),imag(bigD(:)),&
                abs(bigD(:))
    close(291)

  end if

  if ((mod(t,1.d0).lt.dt).and.(bsout.eq."Y")) then !Every 1 a.u. output the basis set to file
    open(436,file="../out/outbs.out") 
    write(436,'(E25.16)')t
    write(436,'(E25.16)')z
    write(436,'(E25.16)')s
    write(436,'(E25.16)')d
    write(436,'(E25.16)')bigD
    close(436)
  end if

  return

end subroutine write_outputs

!-----------------------------------------------------------------------------------------------!

end module outputs
